%% CreatePhysioPredictors_BIDS.m
%
% this script can be used to:
%   1.  Import *.json and *.tsv.gz physiological recordings that follow
%       BIDS standard as well as the corresponding FMR (generated with BV 21.4
%       or newer)
%   2.  Preprocess the cardiac signal derived from a PPU (peripheral pulse unit)
%       and the respiratory signal derived from a breathing belt
%   3.  Perform peak detection on the preprocessed physiological
%       signals and extract different physiological variables (e.g. heart
%       rate, breathing rate, respiratory volume time, etc.)
%   4.  Extract volume- (or slice-) based fMRI triggers from the *.json and
%       *.tsv.gz physiological recording files
%   5.  Extract fMRI aquisition parameters from the FMR and create
%       volume (or slice-based physiological predictors)
%   6.  Extract SDM files with the filtered physiological signals
%   7.  create RETROICOR SDM predictors based on the method proposed by Glover:
%       Glover, G. H., Li, T.-Q., & Ress, D. (2000). Image-based method for
%       retrospective correction of physiological motion effects in fMRI: RETROICOR.
%       Magnetic Resonance in Medicine, 44(1), 162-167.
%       https://doi.org/10.1002/1522-2594(200007)44:1<162::AID-MRM23>3.0.CO;2-E
%   8.  create physiological noise predictors saved in single sdm files,
%       including (shifted) heart rate (HR), HR convolved with the cardiac
%       response function (CRF), breathing rate (BR), respiratory flow
%       (RF), the (shifted) envelope of the respiratory signal (ENV), (shifted) respiration 
%       variation (RV),(shifted) respiration volume per time (RVT) and RV 
%       as well as RVT convolved with the respiratory response function (RRF)
%   9.  save PNG and Matlab figures showing the cardiac and respiratory
%       signal, the identified peaks, the functional scan time and some
%       derived measures
%   10. if specified by the user, compute Pearson correlations of the task
%       design (specified in a SDM file) with the derived physiological
%       regressors
%   11. all results, i.e. the physio structure with all computations
%       (cardiorespiratory.mat), as well as the figures and SDM files are
%       saved in a directory called 'PhysioOut', which is created in the same
%       folder as the specified FMR file
%
% last changed by Judith Eck, 2022-05-11

clear
clc
close all






%    ================================================================================================================================================================
%% 1. save script name, the date the script was changed last and the date the script was run in the ouptut file

p = mfilename('fullpath');
file = dir([p, '.m']);
physio.scriptname = file.name;
physio.scriptversiondate = file.date;          
physio.currentdate = today('datetime');

clear p file






%    ================================================================================================================================================================
%% 2. User-defined Parameters (values are set to reasonable default values)

%   cutoff frequencies in Hz for zero-phase second-order bandpass butterworth
%   filter for the cardiac signal, default values taken from:
%   Elgendi M, Norton I, Brearley M, Abbott D, Schuurmans D. Systolic peak
%   detection in acceleration photoplethysmograms measured from emergency responders
%   in tropical conditions. PLoS ONE. 2013;8(10):76585. doi: 10.1371/journal.pone.0076585
physio.cardiac_low = 0.5;
physio.cardiac_high = 8;

%   Definition of RETROICOR model, effects of cardiac and respiratory cycles estimated as 
%   a linear combination of sinusoidal signals, order of correction terms based on:
%   Harvey, A. K., Pattinson, K. T. S., Brooks, J. C. W., Mayhew, S. D., 
%   Jenkinson, M., & Wise, R. G. (2008). Brainstem functional magnetic resonance 
%   imaging: Disentangling signal from physiological noise. Journal of Magnetic Resonance 
%   Imaging, 28(6), 1337-1344. https://doi.org/10.1002/jmri.21623
physio.RETROICOR.order_c = 3; % number of cardiac harmonics 
physio.RETROICOR.order_r = 4; % number of respiratory harmonics
physio.RETROICOR.order_cr = 1; % number of multiplicative harmonics

%   Threshold for Heart Rate Varibility (a measure of the autonomic nervous activity):
%   RMSDD: root mean square of successive differences between hearbeats in ms
%   Shaffer, F., & Ginsberg, J. P. (2017). An Overview of Heart Rate Variability Metrics and Norms. Frontiers in Public Health, 5, 258. https://doi.org/10.3389/fpubh.2017.00258
%   short-term norms RMSSD: 42(15) M(SD)
physio.RMSDD_thresh = 65;

% Values for outlier removal in HR, based on:
% Kassinopoulos, M., & Mitsis, G. D. (2019). Identification of physiological response functions 
% to correct for fluctuations in resting-state fMRI related to heart rate and respiration. 
% NeuroImage, 202, 116150. https://doi.org/10.1016/j.neuroimage.2019.116150
% used to remove outliers in the resulting Heartrate (HR) signal if there are sudden
% changes in HR due to noise. It needs to be visually inspected whether these sudden changes are indeed noise or true sudden
% changes in the PPU signal
physio.HR_filloutliers_window = 30;     % given in number of samples
physio.HR_filloutliers_ThresholdFactor = 20;     %  normal range between 3-20

%   Shifting of heartate (HR) regressors based on:
%   Shmueli, K., van Gelderen, P., de Zwart, J. A., Horovitz, S. G., Fukunaga, M., 
%   Jansma, J. M., & Duyn, J. H. (2007). Low-frequency fluctuations in the cardiac 
%   rate as a source of variance in the resting-state fMRI BOLD signal. NeuroImage, 
%   38(2), 306-320. https://doi.org/10.1016/j.neuroimage.2007.07.037
physio.HR_shifts = 0:2:24; % 0:2:24 seconds, or -12:6:12 seconds (when referring to Biancardi et al., 2009)

%   Shifting of respiration volume time (RVT) regressors based on:
%   Jo, H. J., Saad, Z. S., Simmons, W. K., Milbury, L. A., & Cox, R. W. (2010). 
%   Mapping sources of correlation in resting state FMRI, with artifact detection and 
%   removal. NeuroImage, 52(2), 571-582. https://doi.org/10.1016/j.neuroimage.2010.04.246
physio.RVT_shifts = 0:5:20; % 0:5:20 seconds, or -24:6:18 seconds (when referring to Biancardi et al., 2009)

%   Shifting of respiration variation (RV) regressors based on:
%   Power, J. D., Plitt, M., Laumann, T. O., & Martin, A. (2017). Sources and implications of whole-brain fMRI 
%   signals in humans. NeuroImage, 146, 609?625. https://doi.org/10.1016/J.NEUROIMAGE.2016.09.038
physio.RV_shifts = -7:7:7; % -7,0,7 seconds

%   Added option of Shifting the respiratory envelope regressor, similar to
%   RV:
physio.ENV_shifts = -7:7:7; % -7,0,7 seconds




%    ================================================================================================================================================================
%% 3. Read all necessary files, i.e. *.json, *.tsv.gz, *.fmr and extract the information needed

%   the Physio.tsv.gz file should take the number of skipped volumes during FMR creation 
%   into account (and hence should not include any triggers for these skipped volumes). 
%   "StartTime" in the Physio.json file should be computed with respect to the first non-skipped volume
%   the last column in the Physio.tsv.gz is expected to contain the trigger
%   loacations (marked with 1)
[jsonfile,jsondir] = uigetfile('*.json', 'Select the JSON File(s) of the Physiorecordings of a Single Run' ,'MultiSelect', 'on');
jsonfile = cellstr(jsonfile);

[tsvgzfile,tsvgzdir] = uigetfile('*.tsv.gz', 'Select the corresponding TSV.GZ File(s) with the Recordings', 'MultiSelect', 'on');
tsvgzfile = cellstr(tsvgzfile);
for i = 1:length(tsvgzfile)
    temptsv(i) = gunzip([tsvgzdir, tsvgzfile{1,i}]);
end
clear i

% FMR: import the fmr file to extract scanning parameters
[fmrifile,fmridir] = uigetfile({'*.fmr'},'Select FMR File of corresponding fMRI dataset');

% TASK SDM: in case you would like to correlate the resulting physiological measures
% with your task predictors, select the task SDM file
[sdmfile,sdmdir] = uigetfile({'*.sdm'},'Select the SDM file of your task design if you would like to correlate your predictors with the physiological measures, if not click Cancel');

% create Physio folder in subfolder of FMR (if it does not exist)
if not(isfolder([fmridir,'/PhysioOut/']))
    mkdir([fmridir,'/PhysioOut/'])
end

% JSON: Import keydata
for i= 1:length(jsonfile)
    alltext = fileread([jsondir jsonfile{i}]);

    strsampling='"SamplingFrequency":'; strstart='"StartTime":'; strcolumns='"Columns":';

    % Save Sampling Frequency
    ind = strfind(alltext, strsampling);
    assert(~isempty(ind), 'The sampling frequency of the data is not defined in the file!');
    physio.Hz(i) = sscanf(alltext(ind(1) + length(strsampling):end), '%g');
    clear ind

    % Save Start Time in variable 'startsec'
    ind = strfind(alltext, strstart);
    assert(~isempty(ind),'The start time of the physiological recording is not defined in the file!');
    physio.startsec(i) = sscanf(alltext(ind(1) + length(strstart):end), '%g');

    % Save Column Headers of the physiological recording in 'colheader'
    ind = strfind(alltext, strcolumns);
    assert(~isempty(ind),'The column headers of the physiological recording saved in the tsv file are not defined!');
    indstart = strfind(alltext(ind(1):end),'['); 
    indend = strfind(alltext(ind(1):end),']');
    colheader = strsplit(alltext(ind(1) + indstart+1 : ind(1)+indend-2), {', ',',','"'});
    colheader = strtrim(colheader);
    physio.colheader(i,:) = colheader(~cellfun('isempty',colheader));
    
    clear alltext ind* str*
end

% TSV.GZ: Import physiological data
for i=1:length(temptsv)
    physio.data{i} = dlmread(temptsv{1,i});
    % make sure that the data is double (for filtering)
    physio.data{i} = double(physio.data{i});
    physio.NoOfTriggerIntervals(i) = sum(diff([0; physio.data{i}(:,end)]) == 1); % n trigger start times
end

% FMR: Import scanning parameters
fid = fopen([fmridir fmrifile]);
FMR = textscan(fid, '%s'); FMR = FMR{1};
physio.TR = str2double(FMR(find(ismember(FMR,'TR:'))+1)); % TR
physio.NoVols = str2double(FMR(find(ismember(FMR,'NrOfVolumes:'))+1)); % n volumes in run
physio.NoSlices = str2double(FMR(find(ismember(FMR,'NrOfSlices:'))+1)); % n slices in run
physio.SliceTimingTableSize = str2double(FMR(find(ismember(FMR,'SliceTimingTableSize:'))+1)); % indicating whether slice acquisition times have been saved in FMR

% Check whether number of slice acquistion times saved in FMR corresponds to the number of slices
if physio.SliceTimingTableSize == physio.NoSlices
    ind = find(ismember(FMR,'SliceTimingTableSize:'));
    physio.SliceTable = str2double(FMR(ind+2 : ind+1+physio.NoSlices)); % save slice acquisition times
else
    disp('No Slice Acquisition Information Saved in Slice Timing Table');
    physio.SliceTable = [];
end

% extract unique entries from the SliceTimeTable (for MB sequences)
physio.SliceTimes = unique(physio.SliceTable);

clearvars -except physio order* *dir *file cardiac* HR* RVT* fill* RMSDD* sdm*

for i = 1:length(physio.startsec)
    % time stamps of physio data in seconds
    time{i} = (physio.startsec(i) : 1/physio.Hz(i) : physio.startsec(i) + 1/physio.Hz(i)*(length(physio.data{i})-1))'; 
    physio.time{i} = time{i}; % save the time stamps of the physio data with 0 being the start of the functional run
    % Determine Nyquist frequency for filtering the data
    nyq_freq(i,:) = physio.Hz(i)/2;
    % extract trigger locations (all ones)
    trigind{i} = find(physio.data{i}(:,end)); 
    % sometimes (e.g for the CMRR Sequence) there are 1s written for the entire time a slice acquisition is on and 
    % not just for the start of a slice/volume, by finding all indices in the last physio.data column where the data 
    % trace changes from 0 to 1, it is possible identify just the beginning of the slice/volume acquisition
    trig_startsli_ind{i} = find(diff([0; physio.data{i}(:,end)]) == 1);
 end

% Check whether both cardiac and respiratory data is available
[pulse_row, pulse_col] = find(contains(physio.colheader, 'card'));
[resp_row, resp_col] = find(contains(physio.colheader, 'resp'));


regr_counter = 0;



%    ================================================================================================================================================================
%% 4. If Cardiac Data exists start to analyze it
if ~isempty(pulse_col)
    
    Fs = physio.Hz(pulse_row);
    time = physio.time{pulse_row};
    Hz_10 = 10; 
    Ts_10 = 1/Hz_10; % sampling steps for 10Hz signal
    time_10 = time(1):Ts_10:time(end); % time vector for 10Hz signal
    cardiac = zscore(physio.data{pulse_row}(:,pulse_col)); % extract z-transformed cardiac signal
        
    % check input signal
    assert(~any(isnan(cardiac)), 'Nan values in the cardiac signal');
    
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 4.1. SYSTOLIC PEAK DETECTION based on:
    % Elgendi M, Norton I, Brearley M, Abbott D, Schuurmans D. Systolic peak detection 
    % in acceleration photoplethysmograms measured from emergency responders in tropical conditions. 
    % PLoS ONE. 2013;8(10):76585. doi: 10.1371/journal.pone.0076585
    % Three-Stage method to get indices of systolic peaks:
    % 1. Preprocessing (bandpass filtering and squaring)
    % 2. Feature extraction (generating potential blocks using two moving averages)
    % 3. Classification (thresholding)
    
    % 1. PREPROCESSING: zero-phase second-order Butterworth filter
    % removing baseline wander and high frequencies not contributing to systolic peaks
    [b, a] = butter(2, [physio.cardiac_low/nyq_freq(pulse_row) physio.cardiac_high/nyq_freq(pulse_row)], 'bandpass'); % order = 2, normalized cut-off frequency between 0 & 1 (1 = nyquist freq)
    
    % changed last by Judith Eck, 17-03-2022 to avoid any weird filter
    % effects
    % concatenate a constant value vector equal to the mean of the signal of about 10% of the signal length to each end
    % to avoid any weird filter effects at the end of the signal
    pad_vec = ones(ceil(length(cardiac)/10),1)*mean(cardiac); % pad vector
    temp = [pad_vec; cardiac; pad_vec];
    cardiac_filt = filtfilt(b,a,temp); 
    cardiac_filt = cardiac_filt(length(pad_vec)+1:end-length(pad_vec));
    
    clear b a; cardiac_filt_plot = zscore(cardiac_filt);
    
    cardiac_filt(cardiac_filt<0) = 0; % clip to ouput signal > 0
    % squaring signal emphasizing large differences from systolic wave and 
    % suppressing small differences from diastolic wave and noise
    cardiac_filt = cardiac_filt.^2;
    
    % 2. FEATURE EXTRACTION: Blocks of interest are generated using 2 moving averages marking systolic and heartbeat areas
    w1 = 0.111; % sec (window size of one systolic peak duration)
    w1_norm = round(w1/(1/Fs)); % n data points in physio for w1
    w2 = 0.667; % in sec (window size of one beat duration)
    w2_norm = round(w2/(1/Fs)); % n data points in physio vector for w2
    MApeak = movmean(cardiac_filt,w1_norm);  % 1st moving average - emphasizing the systolic peak area in the signal
    MAbeat = movmean(cardiac_filt,w2_norm);  % 2nd moving average - emphasizing the beat area to be used as a threshold for the first moving average
    
    % 3. TRHESHOLDING: equation determining offset level 'beta' is based on a brute force search
    beta = 0.02; % offset level
    THR1 = MAbeat + beta*mean(cardiac_filt); % thresholding, MAbeat + a small offset
    % generate block variable with indices that contain possible peaks in cardiac_filt (1 = true), (0 = false)
    blocks = MApeak > THR1;
    % get onsets, offset and durations of blocks of interest
    onset = find(diff(blocks)== 1)+1;     offset = find(diff(blocks)== -1);
    if onset(1) > offset(1) % if the onset of the peak was not recorded
        onset = [1;onset];
    end
    if length(onset) > length(offset) % if the offset of the peak was not recorded
        offset(end+1) = length(blocks);
    end
    duration = offset-onset+1;
    % set second Threshold to size of standard PEAK duration w1_norm
    THR2 = w1_norm;
    % get the indices of the pulse peaks as well as the pulse values
    peaks_ind = [];
    for j = 1 : length(duration)
        if duration(j) >= THR2
            [~,ind] = max(cardiac_filt_plot(onset(j) : offset(j)));
            ind = ind + onset(j)-1;
            peaks_ind = [peaks_ind ind];
        end
    end
    clear j
    physio.PPGpeaks_ind = peaks_ind; % save the indices (within the data vector) of the identified peaks in the physio structure
    physio.PPGpeaks_loc = time(peaks_ind)'; % save the timings of the peaks in seconds
    % get the peaks within the functional scan
    peaks_ind_run = peaks_ind(peaks_ind>=trigind{pulse_row}(1)& peaks_ind<=trigind{pulse_row}(end));
    
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 4.2. HEART RATE and HEART RATE VARIABILITY to cross-check and possibly choose different peak-detection
    HR= 60./diff(physio.PPGpeaks_loc);
    time_HR = [time(1),(diff(physio.PPGpeaks_loc))/2+physio.PPGpeaks_loc(1:end-1),time(end)]; % time is sampled in middle of two consecutive peaks
    HR_raw_Fs = interp1(time_HR,[HR(1),HR,HR(end)],time);
    HR_filloutl_Fs = filloutliers(HR_raw_Fs,'linear','movmedian',round(physio.HR_filloutliers_window*Fs),'ThresholdFactor',physio.HR_filloutliers_ThresholdFactor);
    
    hbi = diff(physio.PPGpeaks_loc);
    physio.hbi_rmssd = sqrt(mean((hbi(2:end) - hbi(1:end-1)).^2))*1000; % calculate RMSSD based on all identified heartbeats
    
    % if Heart Rate Variability expressed as RMSSD is above threshold, plot
    % HR and cardiac signal and let user decide to switch the peak
    % detection method
    if physio.hbi_rmssd > physio.RMSDD_thresh
        figure_tmp = figure('Units', 'Normalized', 'OuterPosition', [0, 0, 0.7, 0.7]);
        ax1 = subplot(2,1,1);
        plotraw = plot(time,cardiac); hold on
        plotfilt =  plot(time,cardiac_filt_plot,'c');
        startscan = plot([time(trigind{pulse_row}(1)) time(trigind{pulse_row}(1))],get(ax1,'YLim'),'Color',[1 0 0]);
        uistack(startscan,'bottom')
        endscan = plot([time(trigind{pulse_row}(end)) time(trigind{pulse_row}(end))],get(ax1,'YLim'),'Color',[1 0 0]);
        uistack(endscan,'bottom')
        plotpeaks = plot(physio.PPGpeaks_loc,cardiac_filt_plot(peaks_ind),'g.', 'MarkerSize', 12);
        title('Cardiac raw signal')
        xlabel('Time (s)')
        ylabel('Normalized amplitude')
        leg = legend([startscan plotraw plotfilt plotpeaks], 'run length', 'cardiac raw data', 'filtered cardiac data', 'peaks');
        set(leg, 'TextColor', [0 0 0],'Location','North', 'FontSize', 8,  'Orientation','horizontal');
    
        ax2 = subplot(2,1,2);
        plot(time,HR_raw_Fs)
        title(sprintf('Heart rate (%d \x00B1 %d bpm), heart rate variability as RMSSD (%d ms)',round(mean(HR_filloutl_Fs)),round(std(HR_filloutl_Fs)), round(physio.hbi_rmssd)))
        xlabel('Time (s)')
        ylabel('Raw heart rate')
        
        warning('The calculated heart rate variability is very high. Please check the cardiac signal in the figure whether this might result from sub-optimal peak-detection');
        
        new = lower(input('Would you like to use an alternative peak detection approach? Y/N [Y]: ','s'));
        if new == 'y'
            % peak detection based on Kassinopoulos et al., 2019
            min_peak = round(0.6*Fs); % normal values for 'minPeak' vary from 0.50 to 0.90 seconds.
            [~,peaks_ind] = findpeaks(cardiac_filt_plot,'MinPeakDistance',min_peak, 'MinPeakProminence', 0.4);
            peaks_ind = peaks_ind'; 
            peaks_ind_run = peaks_ind(peaks_ind>=trigind{pulse_row}(1)& peaks_ind<=trigind{pulse_row}(end));
            physio.PPGpeaks_ind = peaks_ind; 
            physio.PPGpeaks_loc = time(peaks_ind)';
            HR= 60./diff(physio.PPGpeaks_loc);
            time_HR = [time(1),(diff(physio.PPGpeaks_loc))/2+physio.PPGpeaks_loc(1:end-1),time(end)]; % time is sampled in middle of two consecutive peaks
            HR_raw_Fs = interp1(time_HR,[HR(1),HR,HR(end)],time);
            HR_filloutl_Fs = filloutliers(HR_raw_Fs,'linear','movmedian',round(physio.HR_filloutliers_window*Fs),'ThresholdFactor',physio.HR_filloutliers_ThresholdFactor);
    
            hbi = diff(physio.PPGpeaks_loc);
            physio.hbi_rmssd = sqrt(mean((hbi(2:end) - hbi(1:end-1)).^2))*1000; % calculate RMSSD based on all identified heartbeats
        end
        close figure 1
    end


    
    % _______________________________________________________________________________________________________________________________________________________________
    % 4.3. CARDIAC PHASE: compute based on systolic peaks
    cardiac_phase = zeros(length(cardiac),1);
    for t=1:length(cardiac)
        if t < peaks_ind(1) || t >= peaks_ind(end)
            cardiac_phase(t) = NaN;
        else
            prev_peak = find(peaks_ind <=t, 1, 'last');
            t1 = peaks_ind(prev_peak);
            t2 = peaks_ind(prev_peak+1);
            cardiac_phase(t) = 2*pi*(t - t1)/(t2-t1); % phase coded between 0 and 2pi (see Glover et al., 2000)
            clear t1 t2
        end
    end
    clear t
    
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 4.4. HEART RATE computation based on Kassinopoulos et al., 2019
    
    HR_10 = interp1(time,HR_filloutl_Fs,time_10); HR=HR(:);
    physio.HRmean = mean(HR_filloutl_Fs);
    physio.HRstd = std(HR_filloutl_Fs);
    
    % generate shifted versions of the cleaned HR signal (HR_filloutl_Fs) in
    % the original sampling rate
    HR_shiftedfuncs = zeros(length(physio.HR_shifts),length(HR_filloutl_Fs));
    for i = 1:length(physio.HR_shifts)
        shift = round(abs(physio.HR_shifts(i)*Fs));
        if physio.HR_shifts(i) < 0
            HR_shiftedfuncs(i,1:end-shift) =  HR_filloutl_Fs(shift+1:end);
            HR_shiftedfuncs(i,end-shift+1 : end) = mean(HR_filloutl_Fs);
        elseif physio.HR_shifts(i) > 0
            HR_shiftedfuncs(i,1:shift) = mean(HR_filloutl_Fs);
            HR_shiftedfuncs(i,shift+1:end) = HR_filloutl_Fs(1:end-shift);
        else
            HR_shiftedfuncs(i,:) = HR_filloutl_Fs;
        end
    end
    
    HR_final = HR_shiftedfuncs';
    
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 4.5 PLOTTING: plot cardiac measures 
        figure_cardiac = figure('Units', 'Normalized', 'OuterPosition', [0, 0, 0.7, 0.7]);
        ax1 = subplot(2,1,1);
        plotraw = plot(time,cardiac); hold on
        plotfilt =  plot(time,cardiac_filt_plot,'c');
        startscan = plot([time(trigind{pulse_row}(1)) time(trigind{pulse_row}(1))],get(ax1,'YLim'),'Color',[1 0 0]);
        uistack(startscan,'bottom')
        endscan = plot([time(trigind{pulse_row}(end)) time(trigind{pulse_row}(end))],get(ax1,'YLim'),'Color',[1 0 0]);
        uistack(endscan,'bottom')
        plotpeaks = plot(physio.PPGpeaks_loc,cardiac_filt_plot(peaks_ind),'r.', 'MarkerSize', 12);
        plotpeaksrun = plot(time(peaks_ind_run),cardiac_filt_plot(peaks_ind_run),'g.', 'MarkerSize', 12);
        title('Cardiac signal')
        xlabel('Time (s)')
        ylabel('Normalized amplitude')
        leg = legend([startscan plotraw plotfilt plotpeaks plotpeaksrun], 'run length', 'cardiac raw data', 'filtered cardiac data', 'peaks', 'peaks in run');
        set(leg, 'TextColor', [0 0 0],'Location','North', 'FontSize', 8,  'Orientation','horizontal');
    
        ax2 = subplot(2,1,2);
        plot(time,HR_raw_Fs), hold on
        plot(time_10, HR_10)
        title(sprintf('Heart rate (%d \x00B1 %d bpm), heart rate variability as RMSSD (%d ms)',round(mean(HR_filloutl_Fs)),round(std(HR_filloutl_Fs)), round(physio.hbi_rmssd)))
        xlabel('Time (s)')
        leg = legend('raw HR','HR corrected for outliers');
        set(leg, 'TextColor', [0 0 0],'Location','North', 'FontSize', 8,  'Orientation','horizontal');
        
        print(figure_cardiac,'-dpng','-r600',[fmridir, '/PhysioOut/', fmrifile(1:end-4),'_cardiac.png']);
        savefig(figure_cardiac,[fmridir,'/PhysioOut/', fmrifile(1:end-4),'_cardiac.fig']);
        
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 4.6 HR*CRF: Apply standard PRF model (cardiac response function, CRF)
    %  as defined by: Chang, C., Cunningham, J. P., & Glover, G. H. (2009).
    %  Influence of heart rate on the BOLD signal: the cardiac response function.
    %  NeuroImage, 44(3), 857?869. https://doi.org/10.1016/j.neuroimage.2008.09.029
    %  code based on Kassinopoulos et al., 2019
    
    t_IR = 0:Ts_10:60; % time vector for impulse response
    CRF = 0.6*t_IR.^2.7.*exp(-t_IR/1.6)-(16/(sqrt(2*pi*9)))*exp(-(t_IR-12).^2/18); % cardiac response function as defined by Chang et al., 2009
    CRF  = CRF/max(CRF);
    
    HR_10_sm = smooth(HR_10,6*Hz_10); %HR smoothed with a 6 s moving average filter
    
    % in order to avoid the convolution with the zero-padded edges of the data
    % vectors, the approach by the Tapas PhysIO-toolbox is used, using as
    % padding value the mean of the HR
    pad_HR = mean(HR_10_sm);
    HR_pad = [pad_HR * ones(length(CRF)-1, 1); HR_10_sm(:)];
    
    % create HR*CRF 
    HR_conv = conv(HR_pad, CRF, 'valid');
    HR_conv = reshape(HR_conv, size(HR_10_sm));



    % _______________________________________________________________________________________________________________________________________________________________
    % 4.7 VOLUME-BASED REGRESSORS: sample volume-based values of all
    % calculated cardiac measures and save as SDM files
    
    %Sample volume-based cardiac phase, HR, HR*CRF
    
    time_trigger = time(trig_startsli_ind{pulse_row});
    D=abs(time_trigger(:)-time_10(:).'); % get matrix with diff values of size time_triggers x time_10 
    [~,trig_ind_time10] = min(D,[],2);
    
    tmp = physio.NoOfTriggerIntervals(pulse_row)/physio.NoVols;
    if tmp < 1
        disp('There is not for every recorded volume a trigger saved in the TSV file!')
        return
    elseif tmp == 1 % only volume triggers saved in physio.data
        cardiac_filt_vol = cardiac_filt_plot(trig_startsli_ind{pulse_row}); % raw cardiac values
        cardiac_phase_vol = cardiac_phase(trig_startsli_ind{pulse_row}); % phase values
        HR_vol = HR_final(trig_startsli_ind{pulse_row},:); % heart rate values
        HR_conv_vol = HR_conv(trig_ind_time10); % HR*CRF values
    elseif tmp > 1 % slice triggers saved in physio.data
        cardiac_filt_vol = cardiac_filt_plot(trig_startsli_ind{pulse_row}(1:tmp:end));
        cardiac_phase_vol = cardiac_phase(trig_startsli_ind{pulse_row}(1:tmp:end)); % sample the start of the volume
        %cardiac_phase_vol = cardiac_phase(trig_startsli_ind{pulse_row}(round(temp/2):temp:end)); % sample the middle of the volume
        HR_vol = HR_final(trig_startsli_ind{pulse_row}(1:tmp:end),:);
        HR_conv_vol = HR_conv(trig_ind_time10(1:tmp:end));
    end
    clear tmp
        
    % rescale and detrend predictors
    HR_vol = HR_vol./max(HR_vol);
    HR_conv_vol = detrend(HR_conv_vol); HR_conv_vol = HR_conv_vol / max(HR_conv_vol);
        
    % fit Xth order fourier series to estimate cardiac phase
    if physio.RETROICOR.order_c > 0
        for i = 1:physio.RETROICOR.order_c
            dm_phs_c(:,(i*2)-1) = cos(i*cardiac_phase_vol);
            dm_phs_c(:,i*2) = sin(i*cardiac_phase_vol);
        end
        clear i
               
        % Save Cardiac RETROICOR Predictors

        sdm_cp = xff ('new:sdm');
        sdm_cp.NrOfPredictors = (physio.RETROICOR.order_c*2)+1;
        sdm_cp.NrOfDataPoints = physio.NoVols;
        sdm_cp.FirstConfoundPredictor = (physio.RETROICOR.order_c*2)+1;
        counter = 1;
        for i = 1 : 2: sdm_cp.NrOfPredictors-2
            sdm_cp.PredictorNames{i} =['Cardiac_Cos' num2str(counter)];
            sdm_cp.PredictorNames{i+1} = ['Cardiac_Sin' num2str(counter)];
            counter = counter +1;
        end
        clear i
        sdm_cp.PredictorNames{end+1} = 'Constant';
        sdm_cp.SDMMatrix(1:sdm_cp.NrOfDataPoints,1:(physio.RETROICOR.order_c*2)) = dm_phs_c;
        sdm_cp.SDMMatrix(1:sdm_cp.NrOfDataPoints,sdm_cp.NrOfPredictors) = ones(sdm_cp.NrOfDataPoints,1);
        sdm_cp.PredictorColors = [round(linspace(225,30, sdm_cp.NrOfPredictors-1)'),  ones(sdm_cp.NrOfPredictors-1,2)*0];
        sdm_cp.SaveAs([fmridir,'/PhysioOut/',  fmrifile(1:end-4),'_cardiac_RETROICOR.sdm']);
        physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1: regr_counter+(physio.RETROICOR.order_c*2)) = dm_phs_c;
        physio.Regressors.Names(regr_counter+1: regr_counter+(physio.RETROICOR.order_c*2)) = sdm_cp.PredictorNames(1:end-1);
        regr_counter = regr_counter + physio.RETROICOR.order_c*2;
                
        [pulseRetrNaN, ~] = find(isnan(sdm_cp.SDMMatrix));
        pulseRetrNaN = unique(pulseRetrNaN);
        if ~isempty(pulseRetrNaN)
            warning(['NaNs in cardiac RETROICOR predictors in volumes ' num2str(pulseRetrNaN') sprintf('\n')])
        end
        
    end
    
    % save filtered and z-transformed cardiac signal
    sdm_cfilt = xff ('new:sdm');
    sdm_cfilt.NrOfPredictors = 2;
    sdm_cfilt.NrOfDataPoints = physio.NoVols;
    sdm_cfilt.FirstConfoundPredictor = 2;
    sdm_cfilt.PredictorNames{1} = physio.colheader{pulse_row, pulse_col};
    sdm_cfilt.PredictorNames{2} = 'Constant';
    sdm_cfilt.SDMMatrix(1:sdm_cfilt.NrOfDataPoints,1) = cardiac_filt_vol;
    sdm_cfilt.SDMMatrix(1:sdm_cfilt.NrOfDataPoints,sdm_cfilt.NrOfPredictors) = ones(sdm_cfilt.NrOfDataPoints,1);
    sdm_cfilt.PredictorColors = [round(linspace(225,30, sdm_cfilt.NrOfPredictors-1)'),  ones(sdm_cfilt.NrOfPredictors-1,2)*0];
    sdm_cfilt.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_cardiac_BP', num2str(physio.cardiac_low), '-', num2str(physio.cardiac_high), 'Hz_z.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1) = cardiac_filt_vol;
    physio.Regressors.Names(regr_counter+1) = sdm_cfilt.PredictorNames(1:end-1);
    regr_counter = regr_counter + 1;
    
    % save shifted HR predictors
    sdm_hr = xff ('new:sdm');
    sdm_hr.NrOfPredictors = size(HR_vol,2)+1;
    sdm_hr.NrOfDataPoints = physio.NoVols;
    sdm_hr.FirstConfoundPredictor = sdm_hr.NrOfPredictors;
    counter = 0;
    for i = 1 : sdm_hr.NrOfPredictors-1
        sdm_hr.PredictorNames{i} =['HR_shift' num2str(physio.HR_shifts(i))];
    end
    clear i
    sdm_hr.PredictorNames{end+1} = 'Constant';
    sdm_hr.SDMMatrix(1:sdm_hr.NrOfDataPoints,1:sdm_hr.NrOfPredictors-1) = HR_vol;
    sdm_hr.SDMMatrix(1:sdm_hr.NrOfDataPoints,sdm_hr.NrOfPredictors) = ones(sdm_hr.NrOfDataPoints,1);
    sdm_hr.PredictorColors = [round(linspace(225,30, sdm_hr.NrOfPredictors-1)'),  ones(sdm_hr.NrOfPredictors-1,2)*0];
    sdm_hr.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_HR.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1: regr_counter+size(HR_vol,2)) = HR_vol;
    physio.Regressors.Names(regr_counter+1: regr_counter+size(HR_vol,2)) = sdm_hr.PredictorNames(1:end-1);
    regr_counter = regr_counter + size(HR_vol,2);

    % save HR*CRF predictor
    sdm_crf = sdm_cfilt.CopyObject;
    sdm_crf.PredictorNames{1} = 'HR*CRF';
    sdm_crf.SDMMatrix(1:sdm_crf.NrOfDataPoints,1) = HR_conv_vol;
    sdm_crf.PredictorColors = [round(linspace(225,30, sdm_crf.NrOfPredictors-1)'),  ones(sdm_crf.NrOfPredictors-1,2)*0];
    sdm_crf.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_HRCRF.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1) = HR_conv_vol;
    physio.Regressors.Names(regr_counter+1) = sdm_crf.PredictorNames(1:end-1);
    regr_counter = regr_counter + 1;
    

end



%    ================================================================================================================================================================
%% 5. If Respiratory Data exists start to analyze it
if ~isempty(resp_col)

    Fs = physio.Hz(resp_row);
    time = physio.time{resp_row};
    Hz_10 = 10; 
    Ts_10 = 1/Hz_10; % sampling steps for 10Hz signal
    time_10 = time(1):Ts_10:time(end); % time vector for 10Hz signal
    resp = zscore(physio.data{resp_row}(:,resp_col)); % extract z-transformed respiratory signal
        
    % check input signal
    assert(~any(isnan(resp)), 'Nan values in the respiratory signal');
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 5.1. PREPROCESSING of respiratory signal
    % detrend data (adapted from Kassinopoulos et al, 2019)
    resp = detrend(resp,'linear');
    % outlier replacement and filtering adpated from 
    % Power, J. D., Lynch, C. J., Dubin, M. J., Silver, B. M., Martin, A., & Jones, R. M. (2020). 
    % Characteristics of respiratory measures in young adults scanned at rest, including 
    % systematic changes and missed deep breaths. NeuroImage, 204, 116234. https://doi.org/10.1016/j.neuroimage.2019.116234
    % outlier replacement filter to eliminate spurious spike artifacts
    filloutliers_window = 0.25*Fs;
    resp_ro = filloutliers(resp, 'linear', 'movmedian', filloutliers_window); % window centered about the current element and contains window-1 neighboring elements
    % blurring to aid peak detection
    resp_filt = smoothdata(resp_ro, 'sgolay', Fs); % a 1-s window

    
    % _______________________________________________________________________________________________________________________________________________________________
    % 5.2. RESPIRATORY PHASE computation
        % respiratory phase needs to be calculated taking not only the times of peak inspiration into account (peak location), but also the amplitude of inspiration, since the depth of the
        % breathing also influences the amount of head motion
    % 1. amplitude of the respiratory signal from the pneumatic belt, is
    %    normalized to the range (0, Rmax), i.e. find max and min amplitudes and normalize amplitude
         resp_norm = (resp_filt - min(resp_filt)) / (max(resp_filt) - min(resp_filt));
    % 2. Calculate the histogram from the number of occurrences of specific respiratory
    %    amplitudes in bins 1:100 and the bth bin is accordingly centered at bRmax/100
         nbins = 100;
         resp_hist = hist(resp_norm,nbins);
    % 3. Calculate running integral of the histogram, creating an equalized transfer function between the breathing amplitude and respiratory phase,
    %    where end-expiration is assigned a phase of 0 and peak inspiration has a phase of +/-pi. While inhaling the phase spans 0 to pi and during expiration
    %    the phase is negated.   
         resp_transfer_func = [0 (cumsum(resp_hist) / sum(resp_hist))];
         kern_size = round(Fs - 1);
         resp_smooth = conv(resp_norm,ones(kern_size,1),'same'); % smoothed version for taking derivative
         resp_diff = [diff(resp_smooth);0]; % derivative dR/dt
        % for phase calculation +pi was added -> so the adapted range is from 0
        % to 2pi (as defined in the Tapas PhysIO toolbox), for plotting use
        % resp_phase - pi to get the original range defined by Glover
          resp_phase = pi*resp_transfer_func(round(resp_norm * nbins)+1)' .* sign(resp_diff) +pi; % respiratory phase between 0 to 2*pi
    

          
    % _______________________________________________________________________________________________________________________________________________________________
    % 5.3. BREATHING RATE (BR), RESPIRATION VARIATION (RV), RESPIRATION VOLUME PER TIME (RVT),RESPIRATORY FLOW (RF), WINDOWED ENVELOPE OVER THE RESPIRATORY TRACE (ENV) 
    % peak detection adapted from Power, J. D., Lynch, C. J., Dubin, M. J., Silver, B. M., Martin, 
    % A., & Jones, R. M. (2020). Characteristics of respiratory measures in young adults scanned 
    % at rest, including systematic changes and missed deep breaths. NeuroImage, 204, 116234. 
    % https://doi.org/10.1016/j.neuroimage.2019.116234
    % RVT, BR and RF computation adapted from Kassinopoulos et al., 2019
    % RV as defined by: Chang, C., & Glover, G. H. (2009). Relationship between respiration, end-tidal CO2, and 
    % BOLD signals in resting-state fMRI. NeuroImage, 47(4), 1381?1393. https://doi.org/10.1016/j.neuroimage.2009.04.048
    % RV = standard deviation of the respiratory signal over 6 seconds, computation based on Power et al., 2020
    % ENV = windowed envelope of the respiratory signal over a 10-s window adapted from
    % Power et al., 2020

    % z-scoring of filtered respiratory data
    resp_filtz = zscore(resp_filt);
    resp_10 = interp1(time,resp_filtz,time_10);
    
    % calculate ENV = windowed envelope of the respiratory signal over a 10-s window
    ENV = envelope(resp_filtz, round(Fs*10), 'rms'); 
    
    % generate shifted versions of ENV
    ENV_shiftedfuncs = zeros(length(physio.ENV_shifts),length(ENV));
    for i = 1:length(physio.ENV_shifts)
        shift = abs(physio.ENV_shifts(i)*Fs);
        if physio.ENV_shifts(i) < 0
            ENV_shiftedfuncs(i,1:end-shift) = ENV(shift+1:end);
            ENV_shiftedfuncs(i,end-shift+1 : end) = mean(ENV);
        elseif physio.ENV_shifts(i) > 0
            ENV_shiftedfuncs(i,1:shift) = mean(ENV);
            ENV_shiftedfuncs(i,shift+1:end) = ENV(1:end-shift);
        else 
            ENV_shiftedfuncs(i,:) = ENV;
        end
    end

    ENV_final = ENV_shiftedfuncs';
    
    % calculate RV = respiratory variation as standard deviation of the
    % respiratory signal over a 6-s window
    RV = movstd(resp_filtz, round(Fs*6), 'endpoints', 'shrink');
 
     % generate shifted versions of ENV
    RV_shiftedfuncs = zeros(length(physio.RV_shifts),length(RV));
    for i = 1:length(physio.RV_shifts)
        shift = abs(physio.RV_shifts(i)*Fs);
        if physio.RV_shifts(i) < 0
            RV_shiftedfuncs(i,1:end-shift) = RV(shift+1:end);
            RV_shiftedfuncs(i,end-shift+1 : end) = mean(RV);
        elseif physio.RV_shifts(i) > 0
            RV_shiftedfuncs(i,1:shift) = mean(RV);
            RV_shiftedfuncs(i,shift+1:end) = RV(1:end-shift);
        else 
            RV_shiftedfuncs(i,:) = RV;
        end
    end

    RV_final = RV_shiftedfuncs';   
    
    % find peaks and troughs;
    % minpeakdistance = 1.8 seconds, presumes breaths occur more than 1.8 s apart
    [resp_peaks,resp_peaks_ind] = findpeaks(resp_filtz, 'minpeakdistance',Fs*1.8,'minpeakprominence',.5);
    [resp_troughs,resp_troughs_ind] = findpeaks(-resp_filtz, 'minpeakdistance',Fs*1.8,'minpeakprominence',.5);
    resp_troughs = -resp_troughs;
    resp_peaks_ind_run = resp_peaks_ind(resp_peaks_ind>=trigind{resp_row}(1)& resp_peaks_ind<=trigind{resp_row}(end));
    physio.RESPpeaks_ind = resp_peaks_ind';
    physio.RESPpeaks_loc = time(resp_peaks_ind)';
    
    respUpp_10 = interp1([time(1),time(resp_peaks_ind)',time(end)],[resp_peaks(1),resp_peaks',resp_peaks(end)],time_10);
    respLow_10 = interp1([time(1),time(resp_troughs_ind)',time(end)],[resp_troughs(1),resp_troughs',resp_troughs(end)],time_10);
    
    % calculate breathing rate (BR)
    BR = 60./diff(time(resp_peaks_ind));
    time_BR = [time(1),(diff(time(resp_peaks_ind))')/2+time(resp_peaks_ind(1:end-1))',time(end)];
    BR_10 = interp1(time_BR,[BR(1),BR',BR(end)],time_10);
    
    physio.BRmean = mean(BR_10);
    physio.BRstd = std(BR_10);
    
    % calculate respiratory volume per time (RVT) = change in breath amplitude
    % over one breath cycle
    RVT = ((respUpp_10-respLow_10).*BR_10)';
    
    % generate shifted versions of the cleaned RVT signal at 10 Hz
    RVT_shiftedfuncs = zeros(length(physio.RVT_shifts),length(RVT));
    for i = 1:length(physio.RVT_shifts)
        shift = abs(physio.RVT_shifts(i)*Hz_10);
        if physio.RVT_shifts(i) < 0
            RVT_shiftedfuncs(i,1:end-shift) = RVT(shift+1:end);
            RVT_shiftedfuncs(i,end-shift+1 : end) = mean(RVT);
        elseif physio.RVT_shifts(i) > 0
            RVT_shiftedfuncs(i,1:shift) = mean(RVT);
            RVT_shiftedfuncs(i,shift+1:end) = RVT(1:end-shift);
        else 
            RVT_shiftedfuncs(i,:) = RVT;
        end
    end

    RVT_final = RVT_shiftedfuncs';
    

    % calculate respiratory flow (RF)
    resp_s = smooth(resp_10,Hz_10*1.5) ; % using a moving average window of 1.5 s to avoid spike artifacts
    RF = diff(resp_s); RF=[0;RF(:)]; RF = RF.^2;
    
   
    % _______________________________________________________________________________________________________________________________________________________________
    % 5.4 PLOTTING: plot respiratory measures    
    figure_respiratory = figure('Units', 'Normalized', 'OuterPosition', [0, 0, 0.7, 0.7]);
    ax1 = subplot(5,2,1);
    hist(resp_norm,100), title('Histogram of amplitude-normalized respiration')
    ax2 = subplot(5,2,3:4);
    plotresp = plot(time,resp); hold on
    plotfilt = plot(time,resp_filtz,'c');
    plotpeaksresp = plot(time(resp_peaks_ind),resp_filtz(resp_peaks_ind),'r.', 'MarkerSize', 12);
    plotpeaksresprun = plot(time(resp_peaks_ind_run),resp_filtz(resp_peaks_ind_run),'g.', 'MarkerSize', 12);
    y = get(ax2,'YLim');
    ylim(ax2,[y(1) y(2)+2]);
    startscan = plot([time(trigind{resp_row}(1)) time(trigind{resp_row}(1))],get(ax2,'YLim'),'Color',[1 0 0]);
    uistack(startscan,'bottom')
    endscan = plot([time(trigind{resp_row}(end)) time(trigind{resp_row}(end))],get(ax2,'YLim'),'Color',[1 0 0]);
    uistack(endscan,'bottom')
    plot(time_10,respUpp_10)
    plot(time_10,respLow_10)
    title('Respiratory signal')
    ylabel('Normalized amplitude')
    leg = legend([startscan plotresp plotfilt plotpeaksresp plotpeaksresprun], 'run length', 'raw data ', 'filtered data', 'peaks', 'peaks within run');
    set(leg, 'TextColor', [0 0 0],'Location','North', 'FontSize', 8,  'Orientation','horizontal');
    ax3 = subplot(5,2,5:6);
    plot(time_10,BR_10)
    title(sprintf('Breathing rate (BR): %3.1f \x00B1 %3.1f ',mean(BR_10),std(BR_10)))
    ylabel('BR (rpm)')
    ax4 = subplot(5,2,7:8);
    plot(time,resp_filtz), hold on, plot(time_10,zscore(RVT)), plot(time, zscore(ENV)), plot(time, zscore(RV))
    ylabel('Normalized amplitude')
    y = get(ax4,'YLim');
    ylim(ax4,[y(1) y(2)+2]);
    title('Respiration volume per time (RVT), Windowed envelope over respiration signal (ENV), Respiration variation (RV)')
    leg = legend('Respiration','RVT', 'ENV', 'RV');
    set(leg, 'TextColor', [0 0 0],'Location','North', 'FontSize', 8,  'Orientation','horizontal');
    ax5 = subplot(5,2,9:10);
    plot(time_10,RF,'g'),       
    title('Respiratory flow (RF)')
    ylabel('RF (a.u.)')
    xlabel('Time (s)')
    
    print(figure_respiratory,'-dpng','-r600',[fmridir, '/PhysioOut/', fmrifile(1:end-4),'_resp.png']);
    savefig(figure_respiratory,[fmridir,'/PhysioOut/', fmrifile(1:end-4),'_resp.fig']);
        
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 5.5 RVT*RRF and RV*RRF: Apply standard PRF model (respiratory response function, RRF)
    %  as defined by: Birn, R. M., Smith, M. A., Jones, T. B., & Bandettini, P. A. (2008). 
    %  The respiration response function: The temporal dynamics of fMRI signal fluctuations 
    %  related to changes in respiration. NeuroImage, 40(2), 644?654. 
    %  https://doi.org/10.1016/j.neuroimage.2007.11.059
    %  code based on Kassinopoulos et al., 2019
    
    t_IR_10 = 0:Ts_10:60; % time vector for impulse response
    t_IR_Fs = 0:1/Fs:60;
    RRF_10 = 0.6*t_IR_10.^(2.1).*exp(-t_IR_10/1.6)-0.0023*t_IR_10.^3.54.*exp(-t_IR_10/4.25);
    RRF_10  = RRF_10/max(RRF_10);
    RRF_Fs = 0.6*t_IR_Fs.^(2.1).*exp(-t_IR_Fs/1.6)-0.0023*t_IR_Fs.^3.54.*exp(-t_IR_Fs/4.25); 
    RRF_Fs  = RRF_Fs/max(RRF_Fs);
    
    % in order to avoid the convolution with the zero-padded edges of the data
    % vectors, the approach by the Tapas PhysIO-toolbox is used, using as
    % padding value the mean of the RVT
    pad_RVT = mean(RVT);
    RVT_pad = [pad_RVT * ones(length(RRF_10)-1, 1); RVT(:)];
    pad_RV = mean(RV);
    RV_pad = [pad_RV * ones(length(RRF_Fs)-1, 1); RV(:)];
    
    % create RVT*RRF 
    RVT_conv = conv(RVT_pad, RRF_10, 'valid');
    RVT_conv = reshape(RVT_conv, size(RVT));
    RV_conv = conv(RV_pad, RRF_Fs, 'valid');
    RV_conv = reshape(RV_conv, size(RV));

    
    
    % _______________________________________________________________________________________________________________________________________________________________
    % 5.6 VOLUME-BASED REGRESSORS: sample volume-based values of all
    % calculated respiratory measures and save as SDM files
    
    % Sample volume-based filtered respiratory signal, respiratory phase, BR, RVT, RF, ENV, RVT*RRF
    time_trigger = time(trig_startsli_ind{resp_row});
    D=abs(time_trigger(:)-time_10(:).'); % get matrix with diff values of size time_triggers x time_10 
    [~,trig_ind_time10] = min(D,[],2);
    
    tmp = physio.NoOfTriggerIntervals(resp_row)/physio.NoVols;
    if tmp < 1
        disp('There is not for every recorded volume a trigger saved in the TSV file!')
        return
    elseif tmp == 1 % only volume triggers saved in physio.data
        resp_filt_vol = resp_filtz(trig_startsli_ind{resp_row}); % filtered respiratory values
        resp_phase_vol = resp_phase(trig_startsli_ind{resp_row}); % respiratory phase
        BR_vol = BR_10(trig_ind_time10); % breathing rate
        RVT_vol = RVT_final(trig_ind_time10,:); % respiration volume per time
        ENV_vol = ENV_final(trig_startsli_ind{resp_row},:); % respiratory envelope
        RV_vol = RV_final(trig_startsli_ind{resp_row},:); % respiratory variation
        RF_vol = RF(trig_ind_time10); % respiration flow
        RVT_conv_vol = RVT_conv(trig_ind_time10); % RVT*RRF
        RV_conv_vol = RV_conv(trig_startsli_ind{resp_row}); % RV*RRF
    elseif tmp > 1 % slice triggers saved in physio.data
        resp_filt_vol = resp_filtz(trig_startsli_ind{resp_row}(1:tmp:end)); % sample the start of the volume
        resp_phase_vol = resp_phase(trig_startsli_ind{resp_row}(1:tmp:end)); 
        %resp_phase_vol = resp_phase(trig_startsli_ind{resp_row}(round(temp/2):temp:end)); % sample the middle of the volume
        BR_vol = BR_10(trig_ind_time10(1:tmp:end));
        RVT_vol = RVT_final(trig_ind_time10(1:tmp:end),:);
        ENV_vol = ENV_final(trig_startsli_ind{resp_row}(1:tmp:end),:);
        RV_vol = RV_final(trig_startsli_ind{resp_row}(1:tmp:end),:);
        RF_vol = RF(trig_ind_time10(1:tmp:end));
        RVT_conv_vol = RVT_conv(trig_ind_time10(1:tmp:end));
        RV_conv_vol = RV_conv(trig_startsli_ind{resp_row}(1:tmp:end));
    end
    clear tmp
        
    % rescale and detrend predictors
    RVT_vol = RVT_vol./max(RVT_vol); 
    RV_vol = RV_vol./max(RV_vol); 
    RF_vol = RF_vol./max(RF_vol); 
    BR_vol = BR_vol./max(BR_vol); 
    ENV_vol = ENV_vol./max(ENV_vol);
    RVT_conv_vol = detrend(RVT_conv_vol); RVT_conv_vol = RVT_conv_vol / max(RVT_conv_vol); 
    RV_conv_vol = detrend(RV_conv_vol); RV_conv_vol = RV_conv_vol / max(RV_conv_vol); 
    
    % fit Xth order fourier series to estimate respiratory phase
    if physio.RETROICOR.order_r > 0
        for i = 1:physio.RETROICOR.order_r
            dm_phs_r(:,(i*2)-1) = cos(i*resp_phase_vol);
            dm_phs_r(:,i*2) = sin(i*resp_phase_vol);
        end
        clear i
           
        % Save Respiratory RETROICOR Model
        sdm_rp = xff ('new:sdm');
        sdm_rp.NrOfPredictors = (physio.RETROICOR.order_r*2)+1;
        sdm_rp.NrOfDataPoints = physio.NoVols;
        sdm_rp.FirstConfoundPredictor = (physio.RETROICOR.order_r*2)+1;
        counter = 1;
        for i = 1 : 2: sdm_rp.NrOfPredictors-2
            sdm_rp.PredictorNames{i} =['Resp_Cos' num2str(counter)];
            sdm_rp.PredictorNames{i+1} = ['Resp_Sin' num2str(counter)];
            counter = counter +1;
        end
        clear i
        sdm_rp.PredictorNames{end+1} = 'Constant';
        sdm_rp.SDMMatrix(1:sdm_rp.NrOfDataPoints,1:(physio.RETROICOR.order_r*2)) = dm_phs_r;
        sdm_rp.SDMMatrix(1:sdm_rp.NrOfDataPoints,sdm_rp.NrOfPredictors) = ones(sdm_rp.NrOfDataPoints,1);
        sdm_rp.PredictorColors = [ones(sdm_rp.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_rp.NrOfPredictors-1)')];
        sdm_rp.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_resp_RETROICOR.sdm']);
        physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1: regr_counter+(physio.RETROICOR.order_r*2)) = dm_phs_r;
        physio.Regressors.Names(regr_counter+1: regr_counter+(physio.RETROICOR.order_r*2)) = sdm_rp.PredictorNames(1:end-1);
        regr_counter = regr_counter + physio.RETROICOR.order_r*2;
        
        [respRetrNaN, ~] = find(isnan(sdm_rp.SDMMatrix));
        respRetrNaN = unique(respRetrNaN);
        if ~isempty(respRetrNaN)
            warning(['NaNs in respiratory RETROICOR predictors in volumes ' num2str(respRetrNaN') sprintf('\n')])
        end
        
    end
    
    % Save filtered and zscored respiratory Signal
    sdm_rfilt = xff ('new:sdm');
    sdm_rfilt.NrOfPredictors = 2;
    sdm_rfilt.NrOfDataPoints = physio.NoVols;
    sdm_rfilt.FirstConfoundPredictor = 2;
    sdm_rfilt.PredictorNames{1} = physio.colheader{resp_row,resp_col};
    sdm_rfilt.PredictorNames{2} = 'Constant';
    sdm_rfilt.SDMMatrix(1:sdm_rfilt.NrOfDataPoints,1) = resp_filt_vol;
    sdm_rfilt.SDMMatrix(1:sdm_rfilt.NrOfDataPoints,sdm_rfilt.NrOfPredictors) = ones(sdm_rfilt.NrOfDataPoints,1);
    sdm_rfilt.PredictorColors = [ones(sdm_rfilt.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_rfilt.NrOfPredictors-1)')];
    sdm_rfilt.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_resp_filtz.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1) = resp_filt_vol;
    physio.Regressors.Names(regr_counter+1) = sdm_rfilt.PredictorNames(1:end-1);
    regr_counter = regr_counter + 1;
    
    % Save BR predictor
    sdm_br = sdm_rfilt.CopyObject;
    sdm_br.PredictorNames{1} = 'BR';
    sdm_br.SDMMatrix(1:sdm_br.NrOfDataPoints,1) = BR_vol;
    sdm_br.PredictorColors = [ones(sdm_br.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_br.NrOfPredictors-1)')];
    sdm_br.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_BR.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1) = BR_vol;
    physio.Regressors.Names(regr_counter+1) = sdm_br.PredictorNames(1:end-1);
    regr_counter = regr_counter + 1;
    
    % Save RF predictor
    sdm_rf = sdm_rfilt.CopyObject;
    sdm_rf.PredictorNames{1} = 'RF';
    sdm_rf.SDMMatrix(1:sdm_rf.NrOfDataPoints,1) = RF_vol;
    sdm_rf.PredictorColors = [ones(sdm_rf.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_rf.NrOfPredictors-1)')];
    sdm_rf.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_RF.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1) = RF_vol;
    physio.Regressors.Names(regr_counter+1) = sdm_rf.PredictorNames(1:end-1);
    regr_counter = regr_counter + 1;
    
    % Save shifted ENV predictors
    sdm_env = sdm_rfilt.CopyObject;
    sdm_env.NrOfPredictors = size(ENV_vol,2)+1;
    sdm_env.FirstConfoundPredictor = sdm_env.NrOfPredictors;
    for i = 1 : sdm_env.NrOfPredictors-1
        sdm_env.PredictorNames{i} =['ENV_shift' num2str(physio.ENV_shifts(i))];
    end
    clear i
    sdm_env.PredictorNames{end+1} = 'Constant';
    sdm_env.SDMMatrix(1:sdm_env.NrOfDataPoints,1:sdm_env.NrOfPredictors-1) = ENV_vol;
    sdm_env.SDMMatrix(1:sdm_env.NrOfDataPoints,sdm_env.NrOfPredictors) = ones(sdm_env.NrOfDataPoints,1);
    sdm_env.PredictorColors = [ones(sdm_env.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_env.NrOfPredictors-1)')];
    sdm_env.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_ENV.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1: regr_counter+size(ENV_vol,2)) = ENV_vol;
    physio.Regressors.Names(regr_counter+1: regr_counter+size(RV_vol,2)) = sdm_env.PredictorNames(1:end-1);
    regr_counter = regr_counter + size(ENV_vol,2);
    
    % Save shifted RV predictors
    sdm_rv = sdm_rfilt.CopyObject;
    sdm_rv.NrOfPredictors = size(RV_vol,2)+1;
    sdm_rv.FirstConfoundPredictor = sdm_rv.NrOfPredictors;
    for i = 1 : sdm_rv.NrOfPredictors-1
        sdm_rv.PredictorNames{i} =['RV_shift' num2str(physio.RV_shifts(i))];
    end
    clear i
    sdm_rv.PredictorNames{end+1} = 'Constant';
    sdm_rv.SDMMatrix(1:sdm_rv.NrOfDataPoints,1:sdm_rv.NrOfPredictors-1) = RV_vol;
    sdm_rv.SDMMatrix(1:sdm_rv.NrOfDataPoints,sdm_rv.NrOfPredictors) = ones(sdm_rv.NrOfDataPoints,1);
    sdm_rv.PredictorColors = [ones(sdm_rv.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_rv.NrOfPredictors-1)')];
    sdm_rv.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_RV.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1: regr_counter+size(RV_vol,2)) = RV_vol;
    physio.Regressors.Names(regr_counter+1: regr_counter+size(RV_vol,2)) = sdm_rv.PredictorNames(1:end-1);
    regr_counter = regr_counter + size(RV_vol,2);
    
    % Save shifted RVT predictors
    sdm_rvt = sdm_rfilt.CopyObject;
    sdm_rvt.NrOfPredictors = size(RVT_vol,2)+1;
    sdm_rvt.FirstConfoundPredictor = sdm_rvt.NrOfPredictors;
    for i = 1 : sdm_rvt.NrOfPredictors-1
        sdm_rvt.PredictorNames{i} =['RVT_shift' num2str(physio.RVT_shifts(i))];
    end
    clear i
    sdm_rvt.PredictorNames{end+1} = 'Constant';
    sdm_rvt.SDMMatrix(1:sdm_rvt.NrOfDataPoints,1:sdm_rvt.NrOfPredictors-1) = RVT_vol;
    sdm_rvt.SDMMatrix(1:sdm_rvt.NrOfDataPoints,sdm_rvt.NrOfPredictors) = ones(sdm_rvt.NrOfDataPoints,1);
    sdm_rvt.PredictorColors = [ones(sdm_rvt.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_rvt.NrOfPredictors-1)')];
    sdm_rvt.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_RVT.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1: regr_counter+size(RVT_vol,2)) = RVT_vol;
    physio.Regressors.Names(regr_counter+1: regr_counter+size(RVT_vol,2)) = sdm_rvt.PredictorNames(1:end-1);
    regr_counter = regr_counter + size(RVT_vol,2);

    
    % save RVT*RRF predictor
    sdm_rvtrrf = sdm_rfilt.CopyObject;
    sdm_rvtrrf.PredictorNames{1} = 'RVT*RRF';
    sdm_rvtrrf.SDMMatrix(1:sdm_rvtrrf.NrOfDataPoints,1) = RVT_conv_vol;
    sdm_rvtrrf.PredictorColors = [ones(sdm_rvtrrf.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_rvtrrf.NrOfPredictors-1)')];
    sdm_rvtrrf.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_RVTRRF.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1) = RVT_conv_vol;
    physio.Regressors.Names(regr_counter+1) = sdm_rvtrrf.PredictorNames(1:end-1);
    regr_counter = regr_counter + 1;
    
    % save RV*RRF predictor
    sdm_rvrrf = sdm_rfilt.CopyObject;
    sdm_rvrrf.PredictorNames{1} = 'RV*RRF';
    sdm_rvrrf.SDMMatrix(1:sdm_rvrrf.NrOfDataPoints,1) = RV_conv_vol;
    sdm_rvrrf.PredictorColors = [ones(sdm_rvrrf.NrOfPredictors-1,2)*0, round(linspace(225,30, sdm_rvrrf.NrOfPredictors-1)')];
    sdm_rvrrf.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_RVRRF.sdm']);
    physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1) = RV_conv_vol;
    physio.Regressors.Names(regr_counter+1) = sdm_rvrrf.PredictorNames(1:end-1);
    regr_counter = regr_counter + 1;
end


%    ================================================================================================================================================================
%% 6. If Both Cardiac and Respiratory Signal is available add Multiplicative Term to RETROICOR SDM and save as new SDM
% see Harvey et al., 2008:
% interaction between cardiac and respiratory processes, giving rise to
% amplitude modulation of the cardiac signal by the respiratory waveform
% (1. heart rate varies with respiration - known as respiratory sinus
% arrhythmia, 2. venous return to the heart is facilitated during
% inspiration - known as intrathoracic pump)
% see also: Kasper, L., Bollmann, S., Diaconescu, A. O., Hutton, C., Heinzle, J., Iglesias, S., 
% Stephan, K. E. (2017). The PhysIO Toolbox for Modeling Physiological Noise in fMRI Data. 
% Journal of Neuroscience Methods, 276, 56?72. https://doi.org/10.1016/J.JNEUMETH.2016.10.019


% if both cardiac and respiratory predictors exist
if ~isempty(pulse_col) && ~isempty(resp_col)
    % Save Complete RETROICOR Model including multiplicative term
    sdm_crp = sdm_cp.CopyObject;
    sdm_crp.SDMMatrix(:,end : end+ sdm_rp.NrOfPredictors-2) = sdm_rp.SDMMatrix(:,1:end-1);
    sdm_crp.PredictorColors(end:end+sdm_rp.NrOfPredictors-2,:) = sdm_rp.PredictorColors(1:end-1,:);
    sdm_crp.PredictorNames(end : end+sdm_rp.NrOfPredictors-2) = sdm_rp.PredictorNames(1:end-1);
    
    % a first order interaction model results in 4 predictors    
    if physio.RETROICOR.order_cr > 0
        counter =1;
        for i = 1:4:(physio.RETROICOR.order_cr*4)
            dm_phs_cr(:,i) = cos(counter*cardiac_phase_vol + counter*resp_phase_vol);
            dm_phs_cr(:,i+1) = sin(counter*cardiac_phase_vol + counter*resp_phase_vol);
            dm_phs_cr(:,i+2) = cos(counter*cardiac_phase_vol - counter*resp_phase_vol);
            dm_phs_cr(:,i+3) = sin(counter*cardiac_phase_vol - counter*resp_phase_vol);
            counter = counter + 1;
        end
        clear i
        sdm_crp.SDMMatrix(:,end+1 : end + size(dm_phs_cr,2)) = dm_phs_cr;
        for i = 1 : size(dm_phs_cr,2)
            sdm_crp.PredictorNames(end+1) ={['CardiacResp_' num2str(i)]};
        end
        clear i
        sdm_crp.PredictorColors(end+1:end+(physio.RETROICOR.order_cr*4),:) = [ones(physio.RETROICOR.order_cr*4,1)*0, round(linspace(225,30, physio.RETROICOR.order_cr*4)'),ones(physio.RETROICOR.order_cr*4,1)*0];
        
        physio.Regressors.Matrix(1:physio.NoVols,regr_counter+1: regr_counter+(physio.RETROICOR.order_cr*4)) = dm_phs_cr;
        physio.Regressors.Names(regr_counter+1: regr_counter+(physio.RETROICOR.order_cr*4)) = sdm_crp.PredictorNames(end+1-(physio.RETROICOR.order_cr*4):end);
        regr_counter = regr_counter + physio.RETROICOR.order_cr*4;
    end
    sdm_crp.SDMMatrix(:,end+1) = ones(sdm_crp.NrOfDataPoints,1);
    sdm_crp.PredictorNames{end+1} = 'Constant';
    sdm_crp.NrOfPredictors = size(sdm_crp.SDMMatrix,2);
    sdm_crp.FirstConfoundPredictor =  sdm_crp.NrOfPredictors;
    sdm_crp.SaveAs([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_cardiacresp_RETROICOR.sdm']);
end


%    ================================================================================================================================================================
%% 7. Correlate the resulting physiological measures with the task predictors specified in the SDM file
% if the user has specified a task design matrix, Pearson correlations
% between all task predictors and all physio predictors will be calculated
% and saved in the physio structure
    
if ~isequal(sdmfile,0)
   sdm_task = xff([sdmdir sdmfile]);
   % save the design matrix and the corresponding predictor names (without
   % the constant) in the physio structure physio.Task
   physio.Task.Matrix = sdm_task.SDMMatrix(:,1:end-1);
   physio.Task.Names = sdm_task.PredictorNames(1:end-1);
   
   % calcualte the Pearson correlation coefficients and the corresponding
   % p-values and save these with the corresponding predictor name
   % combinations in the physio structure physio.TaskNoiseCorr
   [rho,pval] = corr(physio.Task.Matrix(:,1:end),physio.Regressors.Matrix);
   physio.TaskNoiseCorr.Rho = rho;
   physio.TaskNoiseCorr.PVal = pval;
   
   for t = 1 : numel(physio.Task.Names)
       for n = 1: numel(physio.Regressors.Names)
           physio.TaskNoiseCorr.Name{t,n} = [physio.Task.Names{t} ,', ', physio.Regressors.Names{n}];
           % display a warning message if a significant correlation is
           % found
           if physio.TaskNoiseCorr.PVal(t,n) < 0.05
               warning(['Significant correlation (rho = ' num2str(physio.TaskNoiseCorr.Rho(t,n)) ', pval= ' num2str(round(physio.TaskNoiseCorr.PVal(t,n),4)) ') between ' physio.Task.Names{t} ' and ' physio.Regressors.Names{n}])
           end
       end
   end
   
    % plot the the Pearson correlation coefficients in a heatmap
    figure_TaskNoiseCorr = figure('Units', 'Normalized', 'OuterPosition', [0, 0, 0.5, 0.5]);
    h = heatmap(round(physio.TaskNoiseCorr.Rho',2));
    h.Colormap = hot;
    h.YLabel = 'Noise Regressors';
    h.YDisplayLabels= strrep(physio.Regressors.Names','_','-');
    h.XLabel = 'Task Regressors';
    h.XDisplayLabels = strrep(physio.Task.Names,'_','-');
    h.Title = 'Pearson Correlation between Task and Noise Regressors';
   
    print(figure_TaskNoiseCorr,'-dpng','-r600',[fmridir, '/PhysioOut/', fmrifile(1:end-4),'_TaskNoiseCorr.png']);
    savefig(figure_TaskNoiseCorr,[fmridir,'/PhysioOut/', fmrifile(1:end-4),'_TaskNoiseCorr.fig']);
    
   
end
 

% Save Physio Structure in Mat File
save([fmridir,'/PhysioOut/', fmrifile(1:end-4),'_',[physio.colheader{1:2}], '.mat'],'physio');
    
    