%% Extract_HR_BR_ForPRT.m
%
% this script can be used to:
%   1.  Import the output of the script CreatePhysioPredictors_BIDS.m (_cardiacrespiratory.mat or _respiratorycardiac.mat)
%       and read in the PRT file (output from ReadPRT.m), the PRT should
%       not have more volumes than the corresponding FMR
%   2.  Plot the PRT and save the PRT information in the physio structure.
%       Note: Please avoid any empty PRT conditions!
%   3.  calculate and save the mean (std) heart- and respiratory rate per
%       condition and per condition interval in the physio structure
%calc 
% last changed by Judith Eck, 2021-12-10    



clear
clc
close all



%    ================================================================================================================================================================
%% 1. Import all necessary files and save the PRT in the physio structure

% Load the ouptut .mat file *_cardiacrespiratory.mat from the script CreatePhysioPredictors_BIDS.m
[physiofile,physiodir] = uigetfile({'*_cardiacrespiratory.mat;*_cardiac*.mat;*_respiratory*.mat',...
    'OutputCreatePhysioPredictors_BIDS.m (*_cardiacrespiratory.mat;*_cardiac.mat;*_respiratory.mat'}, 'ouptut mat file *_cardiacrespiratory.mat from CreatePhysioPredictors_BIDS.m');
load([physiodir physiofile]);

% Import the PRT and copy it to temporary variable
physio.prt = ReadPRT; temp = physio.prt; 

for n = 1:temp.NoCond
    % in case the PRT is defined in volumes, convert the condition onsets and offsets to msec to compare them to the peak times defined in msec
    if strncmp(physio.prt.resolution,'Volumes',6)
        temp.Cond(n).CondOnOffsets(:,1) = temp.Cond(n).CondOnOffsets(:,1)*physio.TR - physio.TR;
        temp.Cond(n).CondOnOffsets(:,2) = temp.Cond(n).CondOnOffsets(:,2)*physio.TR;
        
    elseif strcmp(physio.prt.resolution,'msec')
        
    else
        error('Time resolution in PRT should be either Volumes or msec')
    end
end



%    ================================================================================================================================================================
%% 2. save script name, the date the script was changed last and the date the script was run in the ouptut file

% save this script name and the date the script was changed last in the
% output physio structure
p = mfilename('fullpath');
file = dir([p, '.m']);
physio.prt.scriptname = file.name;
physio.prt.scriptdate = file.date;
clear p file

% Save current date in output file
physio.prt.currentdate = today('datetime');



%    ================================================================================================================================================================
%% 3. HEART RATE: compute HR per condition and condition interval and save it in the physio structure

if isfield(physio,'PPGpeaks_loc')
peaktimesms = physio.PPGpeaks_loc*1000; % copy all peak times in ms to another variable


for n = 1:temp.NoCond
    counter = 1;
    for t = 1:size(temp.Cond(n).CondOnOffsets,1)
        % find all peaks within each condition interval and save the identified peak times per interval in the prt structure per
        % condition within PPGPeaks(condition interval).TimesMsec
        physio.prt.Cond(n).PPGPeaks(t).TimesMsec = peaktimesms(peaktimesms >= temp.Cond(n).CondOnOffsets(t,1) & peaktimesms <= temp.Cond(n).CondOnOffsets(t,2));
        % if no peaks were identified within condition interval take closest peak before onset and closest peak after condition offset
        if numel(physio.prt.Cond(n).PPGPeaks(t).TimesMsec) == 0
            physio.prt.Cond(n).PPGPeaks(t).TimesMsec(1) = peaktimesms(find(peaktimesms < temp.Cond(n).CondOnOffsets(t,1), 1, 'last'));
            physio.prt.Cond(n).PPGPeaks(t).TimesMsec(2) = peaktimesms(find(peaktimesms > temp.Cond(n).CondOnOffsets(t,2), 1, 'first'));
            % if only one peak was identified, take the closest peak in time outside the condition interval irrespective of before onset or after condition offset
        elseif numel(physio.prt.Cond(n).PPGPeaks(t).TimesMsec) == 1
            % take difference of the identified peak with all other peak times and sort the absolute differences
            val = sort(abs(peaktimesms-physio.prt.Cond(n).PPGPeaks(t).TimesMsec(1))); 
            % take the second value in the sorted diff list (as the first one is 0) and add or subtract it with the identified peak within the condition interval to find the closest value in time
            physio.prt.Cond(n).PPGPeaks(t).TimesMsec(2) = peaktimesms(find(peaktimesms ==  physio.prt.Cond(n).PPGPeaks(t).TimesMsec(1) + val(2) | peaktimesms == physio.prt.Cond(n).PPGPeaks(t).TimesMsec(1) - val(2), 1, 'first' )); 
            physio.prt.Cond(n).PPGPeaks(t).TimesMsec = sort(physio.prt.Cond(n).PPGPeaks(t).TimesMsec); % sort the two peaks again in ascending order
            clear val
        end
        % compute time differences between consecutive peaks in condition intervals
        physio.prt.Cond(n).PPGPeaks(t).TimesMsecBetweenPeaks = diff(physio.prt.Cond(n).PPGPeaks(t).TimesMsec);
        % compute mean heart rate, i.e. beats per minutes (BPM) in each condition interval
        physio.prt.Cond(n).PPGPeaks(t).MeanHR = mean(60./(physio.prt.Cond(n).PPGPeaks(t).TimesMsecBetweenPeaks/1000));
        physio.prt.Cond(n).PPGPeaks(t).StdHR = std(60./(physio.prt.Cond(n).PPGPeaks(t).TimesMsecBetweenPeaks/1000));
        % save the time between peaks, defined in msec, for all condition intervals in a single vector (for later mean and std computation
        % of the entire condition)
        physio.prt.Cond(n).PulsePeakTimeDiffVector(counter : counter+numel(physio.prt.Cond(n).PPGPeaks(t).TimesMsecBetweenPeaks)-1) = physio.prt.Cond(n).PPGPeaks(t).TimesMsecBetweenPeaks;
        counter = counter + numel(physio.prt.Cond(n).PPGPeaks(t).TimesMsecBetweenPeaks);
    end
    % Compute Mean Heart Rate as well as Standard deviation for each condition
    physio.prt.Cond(n).MeanHR = mean(60./(physio.prt.Cond(n).PulsePeakTimeDiffVector/1000));
    physio.prt.Cond(n).StdHR = std(60./(physio.prt.Cond(n).PulsePeakTimeDiffVector/1000));
end
clear peaktimesms
end



%    ================================================================================================================================================================
%% 4. BREATHING RATE: compute BR per condition and condition interval and save it in the physio structure

if isfield(physio,'RESPpeaks_loc')
peaktimesms = physio.RESPpeaks_loc*1000; % copy all peak times in ms to another variable


for n = 1:temp.NoCond
    counter = 1;
    for t = 1:size(temp.Cond(n).CondOnOffsets,1)
        % find all peaks within each condition interval and save the identified peak times per interval in the prt structure per
        % condition within RespPeaks(condition interval).TimesMsec
        physio.prt.Cond(n).RespPeaks(t).TimesMsec = peaktimesms(peaktimesms >= temp.Cond(n).CondOnOffsets(t,1) & peaktimesms <= temp.Cond(n).CondOnOffsets(t,2));
        % if no peaks were identified within condition interval take closest peak before onset and closest peak after condition offset
        if numel(physio.prt.Cond(n).RespPeaks(t).TimesMsec) == 0
            physio.prt.Cond(n).RespPeaks(t).TimesMsec(1) = peaktimesms(find(peaktimesms < temp.Cond(n).CondOnOffsets(t,1), 1, 'last'));
            physio.prt.Cond(n).RespPeaks(t).TimesMsec(2) = peaktimesms(find(peaktimesms > temp.Cond(n).CondOnOffsets(t,2), 1, 'first'));
            % if only one peak was identified, take the closest peak in time outside the condition interval irrespective of before onset or after condition offset
        elseif numel(physio.prt.Cond(n).RespPeaks(t).TimesMsec) == 1
            % take the difference of the identified peak with all other peak times and sort the absolute differences
            val = sort(abs(peaktimesms-physio.prt.Cond(n).RespPeaks(t).TimesMsec(1))); 
            % take second val in sorted diff list (as the first one is 0) and add or subtract it with the identified peak within the condition interval to find the closest value in time
            physio.prt.Cond(n).RespPeaks(t).TimesMsec(2) = peaktimesms(find(peaktimesms ==  physio.prt.Cond(n).RespPeaks(t).TimesMsec(1) + val(2) | peaktimesms == physio.prt.Cond(n).RespPeaks(t).TimesMsec(1) - val(2), 1, 'first' )); 
            physio.prt.Cond(n).RespPeaks(t).TimesMsec = sort(physio.prt.Cond(n).RespPeaks(t).TimesMsec); % sort the two peaks again in ascending order
            clear val
        end
        % compute time differences between consecutive peaks in condition intverals
        physio.prt.Cond(n).RespPeaks(t).TimesMsecBetweenPeaks = diff(physio.prt.Cond(n).RespPeaks(t).TimesMsec);
        % compute mean respiratory rate
        physio.prt.Cond(n).RespPeaks(t).MeanBR = mean(60./(physio.prt.Cond(n).RespPeaks(t).TimesMsecBetweenPeaks/1000));
        physio.prt.Cond(n).RespPeaks(t).StdBR = std(60./(physio.prt.Cond(n).RespPeaks(t).TimesMsecBetweenPeaks/1000));
        % save time between peaks, defined in msec, for all condition intervals in a single vector (for later mean and std computation
        % of the entire condition)
        physio.prt.Cond(n).RespPeakTimeDiffVector(counter : counter+numel(physio.prt.Cond(n).RespPeaks(t).TimesMsecBetweenPeaks)-1) = physio.prt.Cond(n).RespPeaks(t).TimesMsecBetweenPeaks;
        counter = counter + numel(physio.prt.Cond(n).RespPeaks(t).TimesMsecBetweenPeaks);
    end
    % Compute Mean Respiratory Rate as well as Standard deviation for each condition
    physio.prt.Cond(n).MeanBR = mean(60./(physio.prt.Cond(n).RespPeakTimeDiffVector/1000));
    physio.prt.Cond(n).StdBR= std(60./(physio.prt.Cond(n).RespPeakTimeDiffVector/1000));
end
clear peaktimesms
end


%    ================================================================================================================================================================
%% 6. Save the physio structure in a mat file in the same directory as the input files
save([physiodir, physiofile(1:end-4),'_inclProtocolInfo.mat'],'physio');